import { reactive, ref } from 'vue'
import { AxiosInstance } from 'axios'
import { deepCopy } from '../utils/tools'
import { CrudElType } from '@/vite-env'

/** 组件配置类 */
export class ComponentConfig {
    /** 调试模式 @default false */
    debug!: boolean

    /** 请求对象配置 @default axios.create() */
    axiosService!: AxiosInstance

    /** 数据加载完成后是否立即查询 */
    isAfterMountQuery!: boolean

    /** Props的参数 */
    PropsStore: CrudElType.PropsType = reactive({})

    /** source数据元素拆分 */
    ElementArrs: {
        filter: Record<string, CrudElType.columnsItemType>
        editor: Record<string, CrudElType.columnsItemType>
        column: Record<string, CrudElType.columnsItemType>
        expand: Record<string, CrudElType.columnsItemType>
        columns: Record<string, CrudElType.columnsItemType>
    } = reactive({ filter: {}, editor: {}, column: {}, expand: {}, columns: {} })

    /** 初始化搜索表单的值 */
    initSearchModel: Record<string, any> = reactive({})

    /** 初始化编辑表单的值 */
    initEditModel: Record<string, any> = reactive({})

    /** 要展示的表格列 */
    tableColumn: CrudElType.columnsType = reactive({})

    /** 搜索/新增/编辑的表单状态 */
    FormStore = reactive({
        /** 显示搜索表单 */
        isShowSearchForm: true,
        /** 是查询中或者提交中 */
        isLoading: false,
        /** 表单Label宽度 */
        labelWidth: 120,
        /** 表单抽屉的大小 */
        drawerSize: 500
    })

    /** Table区域的状态 */
    TableStore = reactive({
        /** 表格高度 */
        tableHeight: 500,
        /** 开启多选 */
        isOpenMultipleSelection: false,
        /** 开启展开行 */
        isOpenExpand: false,
        /** 展开行数量 */
        expandColumn: 4,
        /** 开启索引 */
        isOpenIndex: false,
        /** 显示菜单栏 */
        isShowMenu: true,
        /** 菜单栏对齐 */
        menuAlign: 'left',
        /** 菜单栏宽度 */
        menuWidth: 200,
        /** 加载状态 */
        loading: false
    })

    /** 页码状态存储 */
    PageStore = reactive({
        /** 当前页 */
        current: 1,
        /** 分页大小 */
        pageSize: 10,
        /** 总页数 */
        total: 0,
        /** 每页显示个数选择器的选项设置 */
        pageSizes: [10, 20, 50, 100],
        /** 组件布局，子组件名用逗号分隔 */
        layout: '->,total, sizes, prev, pager, next, jumper',
        /** 只有一页时隐藏分页 */
        isHideOnSinglePage: false
    })

    /** 权限字段映射[显示状态控制]（Crud的按钮权限 与 使用时传入的已拥有的权限 做比较） */
    PermissionNames = reactive({
        add: 'add',
        edit: 'edit',
        delete: 'delete',
        deleteFlag: 'deleteFlag'
    })
    /** 拥有的权限 */
    Permissions!: string[]

    /** API接口路由名称 */
    ApiRouteNames = reactive({
        query: 'query',
        add: 'add',
        edit: 'edit',
        delete: 'delete',
        deleteFlag: 'deleteFlag'
    })

    /** 根据BaseUrl、ApiEndpoints生成的具体接口 */
    ApiNames = reactive({
        query: 'baseUrl/query',
        add: 'baseUrl/add',
        edit: 'baseUrl/edit',
        delete: 'baseUrl/delete',
        deleteFlag: 'baseUrl/deleteFlag'
    })

    constructor(InitConfig: Partial<ComponentConfig>, props: ComponentConfig['PropsStore']) {
        // console.log('【6】、组件配置初始化中', props, InitConfig)
        // console.log('【6.1】、组件配置继承自全局配置，调用super({})')

        // super(InitConfig)
        if (Object.getOwnPropertyNames(InitConfig).length !== 0) {
            const { debug, axiosService, isAfterMountQuery, FormStore, TableStore, PageStore, PermissionNames, ApiRouteNames } = InitConfig

            this.debug = debug || false
            this.axiosService = axiosService!
            this.isAfterMountQuery = isAfterMountQuery || true

            Object.assign(this.FormStore, FormStore)
            Object.assign(this.TableStore, TableStore)
            Object.assign(this.PageStore, PageStore)
            Object.assign(this.PermissionNames, PermissionNames)
            Object.assign(this.ApiRouteNames, ApiRouteNames)
        }

        if (Object.getOwnPropertyNames(props).length !== 0) {
            Object.assign(this.PropsStore, props)
            const { source, Permissions, debug, axiosService, isAfterMountQuery, FormStore, TableStore, PageStore, PermissionNames, ApiRouteNames } = this.PropsStore

            this.InitApiNames()
            this.InitElementArrs(source!)
            this.Permissions = Permissions!

            this.debug = debug ? debug : this.debug
            this.axiosService = axiosService ? axiosService : this.axiosService
            this.isAfterMountQuery = isAfterMountQuery ? isAfterMountQuery : this.isAfterMountQuery
            Object.assign(this.FormStore, FormStore)
            Object.assign(this.TableStore, TableStore)
            Object.assign(this.PageStore, PageStore)
            Object.assign(this.PermissionNames, PermissionNames)
            Object.assign(this.ApiRouteNames, ApiRouteNames)
        }
    }

    /** 初始化接口全称 */
    private InitApiNames() {
        const { url } = this.PropsStore
        const { add, edit, delete: del, deleteFlag, query } = this.ApiRouteNames

        this.ApiNames = {
            add: `/${url}/${add}`,
            edit: `/${url}/${edit}`,
            query: `/${url}/${query}`,
            delete: `/${url}/${del}`,
            deleteFlag: `/${url}/${deleteFlag}`
        }
    }

    /** 初始化表单配置参数 */
    private InitElementArrs(source: CrudElType.SourceType) {
        const { filter, editor, column, expand, columns } = source

        // 初始化
        this.ElementArrs = {
            filter: {},
            editor: {},
            column: {},
            expand: {},
            columns: {}
        }

        // 初始化搜索框和编辑框的数据模型
        const filterModel: Record<string, any> = {}
        const editorModel: Record<string, any> = {}

        // 遍历需要过滤的字段，如果 columns 中存在则将其添加到搜索框的 ElementArrs 中
        for (const item of filter) {
            const columnItem = columns[item]
            if (columnItem) {
                filterModel[item] = columnItem.searchDefaultValue
                this.ElementArrs['filter'][item] = columnItem
            } else {
                console.error(`columns 中不存在参数：${item}`)
            }
        }
        // 将搜索框的初始数据模型赋值给当前的数据模型
        Object.assign(this.initSearchModel, deepCopy(filterModel))

        // 遍历需要编辑的字段，如果 columns 中存在则将其添加到编辑框的 ElementArrs 中
        for (const item of editor) {
            const columnItem = columns[item]
            if (columnItem) {
                editorModel[item] = columnItem.addDefaultValue
                this.ElementArrs['editor'][item] = columnItem
            } else {
                console.error(`columns 中不存在参数：${item}`)
            }
        }
        // 将编辑框的初始数据模型赋值给当前的数据模型
        Object.assign(this.initEditModel, deepCopy(editorModel))

        // 如果设置了展开行，则将其添加到展开行的 ElementArrs 中
        if (expand) {
            for (const item of expand) {
                const columnItem = columns[item]
                if (columnItem) {
                    this.ElementArrs['expand'][item] = columnItem
                } else {
                    console.error(`columns 中不存在参数：${item}`)
                }
            }
        }

        // 将表格数据行的字段添加到 ElementArrs 中
        for (const item of column) {
            const columnItem = columns[item]
            this.ElementArrs['column'][item] = columnItem
        }

        // 将所有的字段都添加到 ElementArrs 中
        this.ElementArrs['columns'] = columns
    }
}
